@@ -4,6 +4,7 @@ from ase import units
 from ase.io import read, write
 
 from dptools.simulate.calculator import DeepMD
+from dptools.simulate.lammps_io import MolInput
 from dptools.utils import read_dump, columnize
 from dptools.utils import get_seed as seed
 
@@ -32,6 +33,7 @@ class Simulation:
         elif not isinstance(atoms, list):
             atoms = [atoms]
         self.atoms = atoms
+
         self.graph = graph
         self.type_map = type_map
         self.path = path
@@ -83,7 +85,7 @@ class Simulation:
         Opts = {0: Opt, 1: CellOpt}
         commands = Opts[cell].get_commands(self, nsw=nsw, ftol=ftol)
 
-        self.run(process=False, commands=commands)
+        Simulation.run(self, process=False, commands=commands)
 
         file_out = os.path.join(self.path, "pre_opt.traj")
         write(file_out, self.atoms)
@@ -97,6 +99,50 @@ class Simulation:
         np.save(os.path.join(self.path, name), data)
 
 
+class NewSimulation(Simulation):
+    """
+    Template/example for making new simulation types in lammps.
+
+    Args:
+        setup_arg1: Argument from params.yaml that is passed to setup method
+        setup_arg2: Argument from params.yaml that is passed to setup method
+        param_arg1: Argument from params.yaml that is passed to get_commands method
+        param_arg2: Argument from params.yaml that is passed to get_commands method
+    """
+
+    def setup(self, setup_arg1=None, setup_arg2=None, **kwargs):
+        """
+        Simulation specific method to do any needed setup before running (e.g., cell deformations).
+        Must call self.get_commands() method.
+        """
+
+        # do stuff with setup args here
+
+        # then set commands
+        self.commands = self.get_commands(**kwargs)
+
+    def get_commands(self, **kwargs):
+        self._warn_unused(**kwargs) # warn about any unused kwargs from params.yaml
+
+        # set all simulation lammps commands (besides initial setup, e.g. unit styles)
+        commands = ["run 0"]
+        return commands
+
+    def run(self, file_out=None):
+        """
+        Rarely need subclass run method. Test without it and if Simulation.run fails
+        for whatever reason, write your own run method.
+        """
+        pass
+
+    def process(self, file_out=None):
+        """
+        Only needed if additional proceessing from lammps output is necessary.
+        Also useful for printing results summaries.
+        """
+        pass
+
+
 class SPE(Simulation):
     """Simple single point energy calculation, not much to it."""
     calc_type = "spe"
@@ -138,10 +184,14 @@ class CellOpt(Simulation):
 class NVT(Simulation):
     calc_type = "nvt-md"
 
-    def setup(self, pre_opt=True, **kwargs):
+    def setup(self, pre_opt=False, **kwargs):
         if pre_opt:
             self.pre_opt(200)
 
+        # group 'unconstrained' created by lammps_io.LammpsInput if constraints present
+        # need to specify only unconstrained atoms for velocity command
+        self._unconstrained = "unconstrained" if self.atoms[0].constraints else "all"
+
         self.commands = self.get_commands(**kwargs)
 
     def get_commands(self, steps=10000, timestep=0.5, Ti=298.0, Tf=298.0, equil_steps=1000, write_freq=100, disp_freq=100, **kwargs):
@@ -154,7 +204,7 @@ class NVT(Simulation):
             "run_style verlet",
             "timestep ${dt}",
             # XXX: Add customizable velocity keywords as args?
-            f"velocity all create {Ti} {seed()} rot yes mom yes dist gaussian",
+            f"velocity {self._unconstrained} create {Ti} {seed()} rot yes mom yes dist gaussian",
             f"fix equil all nvt temp {Ti} {Ti} ${{tdamp}}",
             f"run {equil_steps}",
              "unfix equil",
@@ -172,10 +222,14 @@ class NVT(Simulation):
 class NPT(Simulation):
     calc_type = "npt-md"
 
-    def setup(self, pre_opt=True, **kwargs):
+    def setup(self, pre_opt=False, **kwargs):
         if pre_opt:
             self.pre_opt(200, cell=True)
 
+        # group 'unconstrained' created by lammps_io.LammpsInput if constraints present
+        # need to specify only unconstrained atoms for velocity command
+        self._unconstrained = "unconstrained" if self.atoms[0].constraints else "all"
+
         self.commands = self.get_commands(**kwargs)
 
     def get_commands(self, steps=10000, timestep=0.5, Pi=0.0, Pf=0.0, Ti=298.0, Tf=298.0, equil_steps=1000, write_freq=100, disp_freq=100, **kwargs):
@@ -189,7 +243,7 @@ class NPT(Simulation):
             "run_style verlet",
             "timestep ${dt}",
             # XXX: Add customizable velocity keywords as args?
-            f"velocity all create {Ti} {seed()} rot yes mom yes dist gaussian",
+            f"velocity {self._unconstrained} create {Ti} {seed()} rot yes mom yes dist gaussian",
             f"fix equil all npt temp {Ti} {Ti} ${{tdamp}} tri {Pi} {Pi} ${{pdamp}}",
             f"run {equil_steps}",
              "unfix equil",
@@ -321,6 +375,65 @@ class Vib(Simulation):
             print(f"Mode {j}: {abs(f):.2f}{i}")
 
 
+class GCMC(Simulation):
+    """Grand-canonical monte carlo simulation."""
+    calc_type = "gcmc"
+
+    def setup(self, molecule="H2O", pre_opt=False, pre_opt_mol=False, **kwargs):
+        if os.path.isfile(molecule):
+            mol = read(molecule)
+        else:
+            from ase import build
+            mol = build.molecule(molecule)
+        if mol.cell.sum() == 0.0: # need cell or lammps crashes
+            mol.cell = self.atoms[0].cell.copy()
+            mol.center()
+
+        mol_tags = [self.type_map[m.symbol] + 1 for m in mol] # tag types for lammps mol writer
+        mol.set_tags(mol_tags)
+
+        self._mol = mol.get_chemical_formula().lower() # save for naming files
+
+        if pre_opt_mol:
+            optmol = Opt(mol,
+                         self.graph,
+                         self.type_map,
+                         file_out=f"{self._mol}.traj",
+                         path=self.path,
+                         nsw=200,
+                         )
+            optmol.run()
+
+        molin = MolInput(mol, self.type_map, name=self._mol)
+        molin.write() # write lammps molecule file
+
+        if pre_opt:
+            self.pre_opt(200, cell=True)
+
+        self.commands = self.get_commands(**kwargs)
+
+    def get_commands(self, steps=100, n_ex=10, n_mc=10, T=298.0, P=1.0, dmax=1.0, equil_steps=0, write_freq=1, disp_freq=5, **kwargs):
+        self._warn_unused(**kwargs)
+
+        commands = [
+                "compute_modify  thermo_temp dynamic/dof yes",
+                f"molecule mol mol.{self._mol}",
+                f"group {self._mol} molecule 0",
+                f"fix dpgcmc {self._mol} gcmc 1 {n_ex} {n_mc} 0 {seed()} "\
+                        f"{T} 0.0 {dmax} mol mol pressure {P} full_energy",
+                f"thermo {disp_freq}",
+                f"run {equil_steps}",
+                f"dump 1 all custom {write_freq} gcmc.dump id type x y z",
+                f"run {steps}",
+                ]
+
+        return commands
+
+    def process(self, file_out=None):
+        atoms = read_dump("gcmc.dump", self.type_map)
+        write(self.file_out, atoms)
+
+
 Simulations = {
     "spe": SPE,
     "opt": Opt,
@@ -329,4 +442,5 @@ Simulations = {
     "npt-md": NPT,
     "eos": EOS,
     "vib": Vib,
+    "gcmc": GCMC,
 }
