@@ -25,6 +25,7 @@ def _generate_color():
     for color in colors:
         yield color
 
+
 _gen_color = _generate_color()
 def next_color():
     """
@@ -127,6 +128,53 @@ def convert_dump_cell(lammps_cell):
     shift = np.array([xlo, ylo, zlo])
     return cell, shift
 
+
+def tag_groups(atoms, mult=1.0, retag=True):
+    """
+    Takes Atoms object of disconnected groups (e.g. adsorbates in MOF) and assigns a unique tag
+    to each group. Returns list of nested lists with indices corresponding to each tag
+
+    Args:
+        atoms (ase.Atoms): Atoms object to tag.
+        mult (float): Coefficient for adjusting neighbor search distances
+            (ase.neighborlist.natural_cutoffs function).
+        retag (bool): If false, a copy of atoms is used for tagging, leaving the original
+            tags unmodified.
+
+    Returns:
+        groups (dict): dict of all indices and elements corresponding to
+            each group of tagged atoms.
+    """
+    from ase.neighborlist import NeighborList
+    from ase.neighborlist import natural_cutoffs
+    if not retag:
+        atoms = atoms.copy()
+    nl = NeighborList(natural_cutoffs(atoms, mult=mult), self_interaction=False, bothways=True)
+    nl.update(atoms)
+    atoms.set_tags(0) #initialize all tags to 0
+    tag_index = 1
+    groups = {}
+    # Grabs atom from subset, tags all connected atoms
+    # with tag_index until all atoms in subset are tagged. Repeats until no atoms are untagged
+    while 0 in atoms.get_tags():
+        groups[tag_index] = {}
+        possible_atoms = [a.index for a in atoms if a.tag == 0] # all current untagged atoms
+        neighbor_indices = [possible_atoms[0]] # grabs first untagged atom
+        for i in neighbor_indices: # finds all connected atoms, tags, and adds to list of iterables
+            new_atoms = [k for k in list(nl.get_neighbors(i)[0]) if atoms[k].tag == 0]
+            atoms[i].tag = tag_index
+            for j in new_atoms:
+                atoms[j].tag = tag_index
+            neighbor_indices.extend(new_atoms)
+
+        groups[tag_index]["indices"] = [a.index for a in atoms if a.tag == tag_index]
+        symbols = [a.symbol for a in atoms if a.tag == tag_index]
+        groups[tag_index]["elements"] = list(np.unique(symbols))
+        tag_index += 1
+
+    return groups
+
+
 def read_db(db_name, indices):
     """
     Reads ase db and returns entries as list of Atoms objects.
@@ -143,6 +191,7 @@ def read_db(db_name, indices):
         traj = [row.toatoms() for row in db.select()]
     return traj[string2index(indices)]
 
+
 def graph2typemap(graph):
     """
     Determine type_map for a given deepmd model.
@@ -158,6 +207,7 @@ def graph2typemap(graph):
     type_map = {sym: i for i, sym in enumerate(dp.get_type_map())}
     return type_map
 
+
 def read_type_map(type_map_json):
     if isinstance(type_map_json, dict):
         type_map = type_map_json
@@ -175,6 +225,7 @@ def read_type_map(type_map_json):
         raise TypeError(msg)
     return check_type_map(type_map)
 
+
 def check_type_map(type_map_dict):
     # I'm inconsistent with key-value/value-key when writing type_map
     # so check if need to invert dict such that chem symbols are keys
@@ -193,11 +244,13 @@ def typemap2str(type_map_dict):
     tm_str = ",".join([f"{k}:{v}" for k, v in type_map.items()])
     return tm_str
 
+
 def str2typemap(tm_str):
     items = tm_str.split(",")
     type_map = {i.split(":")[0]: int(i.split(":")[-1]) for i in items}
     return type_map
 
+
 def randomize_seed(in_json):
     """
     Take input .json or dict with training parameters and randomize seeds
@@ -220,6 +273,7 @@ def randomize_seed(in_json):
     in_json["training"]["seed"] = seeds[2]
     return in_json
 
+
 def get_seed(max_val=999999, n=1):
     """
     Returns:
@@ -232,9 +286,6 @@ def get_seed(max_val=999999, n=1):
         seed = np.random.randint(max_val) # avoids single item array
     return seed
 
-def shake(atoms, delta=0.01):
-    new_atoms = atoms.copy()
-
 
 def columnize(*data):
     """
@@ -243,6 +294,7 @@ def columnize(*data):
     """
     return np.array(list(data)).T
 
+
 class Converter:
     """
     Class to convert between different ASE/VASP/LAMMPS outputs. Mostly useful if wanting
@@ -307,16 +359,20 @@ class Converter:
             raise NotImplementedError(f"supported types:\t{types}\nharass me for others")
         return ftype
 
-    def convert(self, **kwargs):
-        traj = []
+    def read(self, **kwargs):
+        self.atoms = []
         for i in self.inputs:
             atoms = self.reader(i, index=self.indices, **kwargs)
-            if len(traj) > 0 and len(atoms) > 1:
+            if len(self.atoms) > 0 and len(atoms) > 1:
                 # check to see if first image is identical to last image of previous file
                 # primarily for concatenating MD runs from flex/overrun jobs
-                pos1 = traj[-1].positions
+                pos1 = self.atoms[-1].positions
                 pos2 = atoms[0].positions
                 if (pos1 == pos2).all():
                     atoms = atoms[1:]
-            traj.extend(atoms)
-        write(self.output, traj)
+            self.atoms.extend(atoms)
+
+    def convert(self, **kwargs):
+        if not hasattr(self, "atoms"):
+            self.read(**kwargs)
+        write(self.output, self.atoms)
